use std::net::SocketAddr;

use axum::{body::Body, extract::Request, http, routing::get, Router};
use i18n::message;
use i18n::message_ids::MessageId;
use library::context::Context;
use library::{config, task};
use tower::ServiceBuilder;
use tower_http::trace::TraceLayer;

mod controller;
mod service;
mod tasks;

/// 启动服务
pub async fn serve() {
    // 初始化日志、数据库连接池、内存缓存；初始化顺序不可变更
    let (_std_guard, _file_guard) = library::core::logger::init_log();
    library::core::db::init_database().await;
    library::cache::inner_cache::init_cache().await;

    let addr = format!("0.0.0.0:{}", config!().server.port);
    let listener = tokio::net::TcpListener::bind(&addr).await.unwrap();
    tracing::info!("服务监听地址: {}", addr);

    // 启动任务
    task::start().await;
    // 启动应用服务
    axum::serve(listener, init().into_make_service_with_connect_info::<SocketAddr>())
        .await
        .unwrap();
}

/// 初始化router，包括router中间件和数据
fn init() -> Router {
    let auth: Router = library::typed_router::get_router();

    let trace_layer = TraceLayer::new_for_http().make_span_with(|request: &Request<Body>| {
        let req_id = match request
            .headers()
            .get("x-request-id")
            .and_then(|value| value.to_str().ok())
        {
            Some(v) if v.chars().all(|c| c.is_alphanumeric()) => v.to_string(), // 确保请求ID只包含字母和数字
            _ => panic!("无效的请求ID"),
        };
        tracing::error_span!("request_id", id = req_id)
    });

    // 配置路由
    // layer之间存在顺序依赖，勿改。layer执行顺序和配置顺序一致
    // fallback路由放到最后，如果无匹配的路由，则不会执行layer，直接返回404
    Router::new()
        .route("/", get(hello))
        .nest(&config!().server.prefix_url, auth)
        .layer(
            ServiceBuilder::new()
                .layer(axum::middleware::from_fn(
                    library::middleware::req_id::handle,
                ))
                .layer(trace_layer)
                .layer(axum::middleware::from_fn(library::middleware::cors::handle))
                .layer(axum::middleware::from_fn(
                    library::middleware::req_log::handle,
                ))
                .layer(axum::middleware::from_fn(
                    library::middleware::req_ctx::authenticate_ctx,
                )),
        )
        .fallback(|| async { (http::status::StatusCode::NOT_FOUND, "Not Found") })
}

pub async fn hello(context: Context) -> String {
    tracing::info!("hello");
    message!(context.get_lang_tag(), MessageId::Hello, "川岳").to_string()
}
